package info.textgrid.middleware.test.online;

import static org.junit.Assert.assertTrue;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import javax.ws.rs.core.Response;
import org.apache.cxf.helpers.IOUtils;
import org.apache.cxf.jaxrs.client.Client;
import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.apache.http.HttpStatus;
import info.textgrid.middleware.OAIPMHProducer;

/**
 * <p>
 * Some online tests for the TextGrid OAIMPH service.
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 */
public class OAIPMHUtilitiesOnline {

  // **
  // FINALS
  // **

  protected static final String OAI_DC_PREFIX = "oai_dc";
  protected static final String OAI_DATACITE_PREFIX = "oai_datacite";
  protected static final String OAI_IDIOMMETS_PREFIX = "oai_idiom_mets";

  protected static final String EXPECTED_OAIDC_FORMAT_CONTENT = "<oai_dc:dc>";
  protected static final String EXPECTED_IDIOMMETS_FORMAT_CONTENT = "<mets ";
  protected static final String EXPECTED_DATACITE_FORMAT_CONTENT = "<datacite:resource>";

  protected static final String NO_SET = null;
  protected static final String ERROR = " >>> ERROR";
  protected static final String OK = " >>> OKIDOKI";
  protected static final String TESTING = "\n >>> TESTING ";
  protected static final String NO_THREAD_NAME = "";
  protected static final String NOT_TESTED = " >>> NOT_TESTED";
  protected static final String NO_TOKEN = "-1";

  protected static final String VERB_LIST_IDENTIFIERS = "ListIdentifiers";
  protected static final String VERB_LIST_RECORDS = "ListRecords";
  protected static final String VERB_GET_RECORD = "GetRecord";
  protected static final String VERB_IDENTIFY = "Identify";
  protected static final String VERB_LIST_METADATA_FORMATS = "ListMetadataFormats";
  protected static final String VERB_LIST_SETS = "ListSets";
  protected static final boolean NO_METADATA_FORMAT_WITH_RESTOK = false;
  protected static final boolean METADATA_FORMAT_WITH_RESTOK = true;
  protected static final String NO_FROM = null;
  protected static final String NO_UNTIL = null;
  // Time to wait between the single queries using resumption tokens in milliseconds.
  protected static final Long TIME = 275l;

  protected static final String HDL_PREFIX = "hdl:";
  protected static final String STAR_PREFIX = "***";

  protected static final long OAIPMH_CLIENT_TIMEOUT = 120000;

  /**
   * @param theClient
   * @param thePath
   * @return
   * @throws IOException
   */
  protected static Response getHttpResponse(Client theClient, String thePath) throws IOException {
    return getHttpResponse(theClient, thePath, "");
  }

  /**
   * @param theClient
   * @param thePath
   * @param theThreadName
   * @return
   * @throws IOException
   */
  protected static Response getHttpResponse(Client theClient, String thePath, String theThreadName)
      throws IOException {

    Response result;

    try {
      Thread.sleep(TIME);
    } catch (InterruptedException e) {
      // Nothing interrupts here!
    }

    WebClient w = WebClient.fromClient(theClient).replaceQuery(thePath);

    System.out.println("\t" + theThreadName + "HTTP GET " + w.getCurrentURI());

    result = w.get();

    // Check HTTP status.
    int status = result.getStatus();

    if (status != HttpStatus.SC_OK) {
      System.err.println("\t" + theThreadName + "status: " + status);
      System.err.println(ERROR);
      assertTrue(false);
    } else {
      System.out.println("\t" + theThreadName + "status: " + status);
    }

    return result;
  }

  /**
   * @param theMillis
   * @return
   */
  public static String getDurationInSecs(long theMillis) {

    int SECS_IN_MILLIS = 1000;
    int secs = SECS_IN_MILLIS;

    return theMillis / secs + " second" + ((theMillis / secs) != 1 ? "s" : "");
  }

  /**
   * <p>
   * Loads a resource.
   * </p>
   * 
   * TODO Put together with the method in TGCrudServiceUtilities! Maybe create a first build maven
   * module for utility things.
   * 
   * @param {@link String} The resource to search for.
   * @return {@link File} The resource.
   * @throws IOException
   */
  protected static File getResource(String resPart) throws IOException {

    File res;

    // If we have an absolute resPart, just return the file.
    if (resPart.startsWith(File.separator)) {
      return new File(resPart);
    }

    URL url = ClassLoader.getSystemClassLoader().getResource(resPart);
    if (url == null) {
      throw new IOException("Resource '" + resPart + "' not found");
    }
    try {
      res = new File(url.toURI());
    } catch (URISyntaxException ue) {
      res = new File(url.getPath());
    }

    return res;
  }

  /**
   * @param theProperty
   * @return
   */
  protected static List<String> getListFromProperties(String theProperty) {

    List<String> result = new ArrayList<String>();

    String parts[] = theProperty.split(",");
    for (String part : parts) {
      result.add(part.trim());
    }

    return result;
  }

  /**
   * @return
   * @throws IOException
   * @throws FileNotFoundException
   */
  protected static Properties getPropertiesFromFile(String theConfigFile)
      throws FileNotFoundException, IOException {

    Properties result = null;

    // Load properties file.
    result = new Properties();
    result.load(new FileInputStream(getResource(theConfigFile)));

    System.out.println("Properties file: " + theConfigFile);
    result.list(System.out);

    return result;
  }

  /**
   * @param theEndpoint
   * @return
   */
  protected static Client getOAIPMHWEebClient(String theEndpoint) {

    Client result = null;

    // Get OAI-PMH REST endpoint and HTTP client.
    System.out.println("Getting OAI-PMH HTTP client --> " + theEndpoint + " <--");

    // Get proxy first, set policy.
    OAIPMHProducer JAXRSClient = JAXRSClientFactory.create(theEndpoint, OAIPMHProducer.class);
    HTTPConduit conduit = WebClient.getConfig(JAXRSClient).getHttpConduit();
    HTTPClientPolicy policy = new HTTPClientPolicy();
    policy.setReceiveTimeout(OAIPMHUtilitiesOnline.OAIPMH_CLIENT_TIMEOUT);
    conduit.setClient(policy);

    // Create Web Client from Web Proxy.
    result = WebClient.client(JAXRSClient);

    return result;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * TODO Generalise in OaiPmhTestUtilities!
   * 
   * @param theClient
   * @param theVerb
   * @param theSet
   * @param theMetadataPrefix
   * @param maxNumberOfPagesToTest
   * @param recordsExpectedPerRequest
   * @param theThreadName
   * @param from
   * @param until
   * @param resumptionTokenANDMetadataPrefix
   * @throws IOException
   */
  protected static void testList(Client theClient, final String theVerb, final String theSet,
      final String theMetadataPrefix, final int maxNumberOfPagesToTest,
      final int recordsExpectedPerRequest, final String theThreadName, final String from,
      final String until, final boolean resumptionTokenANDMetadataPrefix) throws IOException {

    String threadName = tn(theThreadName);

    long startTime = System.currentTimeMillis();

    String testOccurance = "header";
    if (theVerb.equals(OAIPMHUtilitiesOnline.VERB_LIST_RECORDS)) {
      testOccurance = "record";
    }

    String path = "verb=" + theVerb + "&metadataPrefix=" + theMetadataPrefix;

    if (theSet != null) {
      path += "&set=" + theSet;
    }

    if (from != null && until != null) {
      path += "&from=" + from + "&until=" + until;
    }

    Response httpResponse = OAIPMHUtilitiesOnline.getHttpResponse(theClient, path, threadName);
    int status = httpResponse.getStatus();

    int loopCount = 1;
    long timeRunning = System.currentTimeMillis() - startTime;
    System.out.println("\t" + threadName + "time: "
        + OAIPMHUtilitiesOnline.getDurationInSecs(timeRunning) + " (loop " + loopCount
        + (maxNumberOfPagesToTest != 0 ? "/" + maxNumberOfPagesToTest : "") + ")");

    String responseString = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());

    // Test resumption token tags.
    String restok =
        examineResumptionTokenTag(responseString, testOccurance, OAIPMHUtilitiesOnline.NO_TOKEN,
            recordsExpectedPerRequest, loopCount, threadName, theMetadataPrefix);

    // Test general metadata content (if verb is listRecords!), must go conform with the metadata
    // prefix setting.
    if (theVerb.equals(OAIPMHUtilitiesOnline.VERB_LIST_RECORDS)) {
      examineContent(responseString, theMetadataPrefix);
    }

    while (status == HttpStatus.SC_OK && !restok.equals(OAIPMHUtilitiesOnline.NO_TOKEN)) {
      loopCount += 1;
      synchronized (threadName) {
        if (maxNumberOfPagesToTest > 0 && loopCount > maxNumberOfPagesToTest) {
          System.out
              .println("\t" + threadName + "TESTING ONLY " + maxNumberOfPagesToTest + " pages!");
          break;
        }
        path = "verb=" + theVerb + "&resumptionToken=" + restok;
        if (resumptionTokenANDMetadataPrefix) {
          path += "&metadataPrefix=" + theMetadataPrefix;
        }
        httpResponse = OAIPMHUtilitiesOnline.getHttpResponse(theClient, path, threadName);
        timeRunning = System.currentTimeMillis() - startTime;
        System.out.println("\t" + threadName + "time: "
            + OAIPMHUtilitiesOnline.getDurationInSecs(timeRunning) + " (loop " + loopCount
            + (maxNumberOfPagesToTest != 0 ? "/" + maxNumberOfPagesToTest : "") + ")");

        // Test resumption token tags.
        responseString = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());
        restok = examineResumptionTokenTag(responseString, testOccurance, restok,
            recordsExpectedPerRequest, loopCount, threadName, theMetadataPrefix);

        // Test general metadata content (if verb is listRecords!), must go conform with the
        // metadata // prefix setting.
        if (theVerb.equals(OAIPMHUtilitiesOnline.VERB_LIST_RECORDS)) {
          examineContent(responseString, theMetadataPrefix);
        }
      }
    }

    // Only check for max loops, if we do have less loops and no resumption token, it will be fine,
    // too!
    if (!restok.equals(OAIPMHUtilitiesOnline.NO_TOKEN) && loopCount < maxNumberOfPagesToTest) {
      System.out.println(threadName + OAIPMHUtilitiesOnline.ERROR + ": Must have done "
          + maxNumberOfPagesToTest + " loops, but did only " + loopCount);
      assertTrue(false);
    }

    // Check for single loop, we need MORE!
    if (loopCount == 1) {
      System.out
          .println(threadName + OAIPMHUtilitiesOnline.ERROR + ": Must have more than one loop!");
      assertTrue(false);
    }

    System.out.println("\t" + threadName + OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @param theResponseString
   * @param recordOrHeader
   * @param oldtok
   * @param recordsExpectedPerRequest
   * @param loopCount
   * @param theThreadName
   * @param theMetadataPrefix
   * @return Resumption token string, if existing, -1 if either no <resumptionToken> tag is
   *         existing, tag is existing and has no token value.
   * @throws IOException
   */
  protected static String examineResumptionTokenTag(final String theResponseString,
      final String recordOrHeader, final String oldtok, final int recordsExpectedPerRequest,
      final int loopCount, final String theThreadName, final String theMetadataPrefix)
      throws IOException {

    // Test for OAIPMH errors.
    if (theResponseString.contains("<error code=\"badArgument\">")) {
      String message =
          theThreadName + OAIPMHUtilitiesOnline.ERROR + " IN OAIPMH RESPONSE: "
              + theResponseString;
      System.out.println(message);
      throw new IOException(message);
    }

    // Count response objects at first.
    int recordCount = 0;
    int i = theResponseString.indexOf("<" + recordOrHeader + ">", 0);
    while (i != -1) {
      recordCount++;
      i++;
      i = theResponseString.indexOf("<" + recordOrHeader + ">", i);
    }

    System.out.println("\t" + theThreadName + recordOrHeader + "s: " + recordCount);

    // Check if token tag is existing.
    int tokStart = theResponseString.indexOf("<resumptionToken");
    int tokEnd = theResponseString.indexOf("</resumptionToken");

    if (tokStart == -1 && tokEnd == -1) {
      System.out.println("\t" + theThreadName + "token: no token");

      return OAIPMHUtilitiesOnline.NO_TOKEN;
    }

    String restokTmp = theResponseString.substring(tokStart, tokEnd);
    // Get token tag.
    String toktag = restokTmp.substring(0, restokTmp.indexOf(">") + 1).trim();
    System.out.println("\t" + theThreadName + "tokentag: " + toktag);

    // Get token.
    String restok = restokTmp.substring(restokTmp.indexOf(">") + 1).trim();
    System.out.println("\t" + theThreadName + "token: " + restok);

    // Check if old and new token are equal or not.
    boolean tokchanged = !oldtok.equals(restok);
    System.out.println("\t" + theThreadName + "tokchngd: " + tokchanged);

    // Get completeListSize and cursor.
    String sizeStr = toktag.substring(toktag.indexOf("completeListSize=\"") + 18);
    int size = Integer.parseInt(sizeStr.substring(0, sizeStr.indexOf("\"")));
    String cursorStr = toktag.substring(toktag.indexOf("cursor=\"") + 8);
    int cursor = Integer.parseInt(cursorStr.substring(0, cursorStr.indexOf("\"")));
    System.out.println("\t" + theThreadName + "size: " + size + " / " + cursor);

    // If token is provided, test cursor and element count!
    if (!restok.isEmpty()) {
      synchronized (TestGetRecordOnline.class) {
        // Check <record> or <header> count, must be recordsExpectedPerRequest!
        if (recordCount != recordsExpectedPerRequest) {
          String message =
              OAIPMHUtilitiesOnline.ERROR + ": " + recordOrHeader + " count mismatch, must be "
                  + recordsExpectedPerRequest + " if token is provided, but is " + recordCount;
          assertTrue(message, false);
        }
        if (size <= recordsExpectedPerRequest) {
          String message =
              OAIPMHUtilitiesOnline.ERROR + ": completeListSize count mismatch, must be > "
                  + recordsExpectedPerRequest + " if token is provided, but is " + size;
          assertTrue(message, false);
        }
        if (cursor != recordsExpectedPerRequest * loopCount) {
          String message = OAIPMHUtilitiesOnline.ERROR + ": cursor must be "
              + (loopCount * recordsExpectedPerRequest) + " in loop " + loopCount + ", but is "
              + cursor;
          assertTrue(message, false);
        }
      }
    }

    // If no token is provided, stop querying.
    else {
      // Check <record> count, must be completeListSize % recordsExpectedPerRequest.
      if (recordCount != size % recordsExpectedPerRequest) {
        System.err.println(theThreadName + OAIPMHUtilitiesOnline.ERROR + ": " + recordOrHeader
            + " count mismatch, should be " + size % recordsExpectedPerRequest + ", but is "
            + recordCount);
        assertTrue(false);
      }

      // No resumption token available in response.
      return OAIPMHUtilitiesOnline.NO_TOKEN;
    }

    return restok;
  }

  /**
   * @param theThreadName
   * @return
   */
  protected static String tn(String theThreadName) {

    String result = theThreadName;

    if (theThreadName != null && !theThreadName.equals("") && !theThreadName.endsWith(" ")) {
      result = "[" + theThreadName + "] ";
    }

    return result;
  }

  /**
   * @param theResponseString
   * @param theMetadataFormat
   */
  protected static void examineContent(String theResponseString, String theMetadataFormat) {

    // Check for correct metadata content according to metadata prefix.
    if (theMetadataFormat.equals(OAIPMHUtilitiesOnline.OAI_DC_PREFIX)
        && !theResponseString
            .contains("metadataPrefix=\"" + OAIPMHUtilitiesOnline.OAI_DC_PREFIX + "\"")
        && !theResponseString.contains(OAIPMHUtilitiesOnline.EXPECTED_OAIDC_FORMAT_CONTENT)) {
      System.out
          .println(OAIPMHUtilitiesOnline.OAI_DC_PREFIX + " needs to deliver content with schema: "
              + OAIPMHUtilitiesOnline.EXPECTED_OAIDC_FORMAT_CONTENT + "!");

      System.out.println("ERROR:\n" + theResponseString);

      assertTrue(false);
    } else if (theMetadataFormat.equals(OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX)
        && !theResponseString.contains(
            "metadataPrefix=\"" + OAIPMHUtilitiesOnline.EXPECTED_IDIOMMETS_FORMAT_CONTENT
                + "\"")
        && !theResponseString
            .contains(OAIPMHUtilitiesOnline.EXPECTED_IDIOMMETS_FORMAT_CONTENT)) {
      System.out.println(
          OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX + " needs to deliver content with schema: "
              + OAIPMHUtilitiesOnline.EXPECTED_IDIOMMETS_FORMAT_CONTENT + "!");

      System.out.println("ERROR:\n" + theResponseString);

      assertTrue(false);
    } else if (theMetadataFormat.equals(OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX)
        && !theResponseString
            .contains("metadataPrefix=\"" + OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX + "\"")
        && !theResponseString.contains(OAIPMHUtilitiesOnline.EXPECTED_DATACITE_FORMAT_CONTENT)) {
      System.out.println(
          OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX + " needs to deliver content with schema: "
              + OAIPMHUtilitiesOnline.EXPECTED_DATACITE_FORMAT_CONTENT + "!");

      System.out.println("ERROR:\n" + theResponseString);

      assertTrue(false);
    } else {
      System.out.println(theResponseString);
    }
  }

  /**
   * @param theResponseString
   * @param recordOrHeader
   * @param oldtok
   * @param theMetadataPrefix
   * @return Resumption token string, if existing, -1 if either no <resumptionToken> tag is
   *         existing, tag is existing and has no token value.
   * @throws IOException
   */
  protected static String examineResumptionTokenTag(String theResponseString, String recordOrHeader,
      String oldtok, String theMetadataPrefix) throws IOException {

    // Default entries per page are 100 (IDIOM: 30).
    // momentarily.
    int expectedCount = 100;
    if (theMetadataPrefix.equals(OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX)) {
      expectedCount = 30;
    }

    // Test for OAIPMH errors.
    if (theResponseString.contains("<error code=\"badArgument\">")) {
      System.err.println(OAIPMHUtilitiesOnline.ERROR + " IN OAIPMH RESPONSE: " + theResponseString);
      assertTrue(false);
    }

    // Count response objects at first.
    int recordCount = 0;
    int i = theResponseString.indexOf("<" + recordOrHeader + ">", 0);
    while (i != -1) {
      recordCount++;
      i++;
      i = theResponseString.indexOf("<" + recordOrHeader + ">", i);
    }

    System.out.println("\t" + recordOrHeader + "s: " + recordCount);

    // Check if token tag is existing.
    int tokStart = theResponseString.indexOf("<resumptionToken");
    int tokEnd = theResponseString.indexOf("</resumptionToken");

    if (tokStart == -1 && tokEnd == -1) {
      System.out.println("\ttoken: no token");

      return "-1";
    }

    String restokTmp = theResponseString.substring(tokStart, tokEnd);
    // Get token tag.
    String toktag = restokTmp.substring(0, restokTmp.indexOf(">") + 1)
        .trim();
    System.out.println("\ttokentag: " + toktag);

    // Get token.
    String restok = restokTmp.substring(restokTmp.indexOf(">") + 1).trim();
    System.out.println("\ttoken: " + restok);

    // Check if old and new token are equal or not.
    boolean tokchanged = !oldtok.equals(restok);
    System.out.println("\ttokchngd: " + tokchanged);

    // Get completeListSize and cursor.
    String sizeStr = toktag
        .substring(toktag.indexOf("completeListSize=\"") + 18);
    int size = Integer
        .parseInt(sizeStr.substring(0, sizeStr.indexOf("\"")));
    String cursorStr = toktag.substring(toktag.indexOf("cursor=\"") + 8);
    int cursor = Integer
        .parseInt(cursorStr.substring(0, cursorStr.indexOf("\"")));
    System.out.println("\tsize: " + size + " / " + cursor);

    // If token is provided, and we have less than 100 (IDIOM: 30) elements: mekkern!
    if (!restok.isEmpty()) {
      synchronized (TestDHOAIPMHOnline.class) {
        // Check <record> or <header> count, must be 100 (IDIOM: 30)!
        if (recordCount != expectedCount) {
          System.err.println(
              OAIPMHUtilitiesOnline.ERROR + ": " + recordOrHeader + " count mismatch, must be "
                  + expectedCount + " if token is provided, but is " + recordCount);
        }
        if (size <= expectedCount) {
          System.err
              .println(OAIPMHUtilitiesOnline.ERROR + ": completeListSize count mismatch, must be > "
                  + expectedCount + " if token is provided, but is " + size);
        }
        if (recordCount != expectedCount || size <= expectedCount) {
          assertTrue(false);
        }
      }
    }

    // If no token is provided, stop querying.
    else {
      // Check <record> count, must be completeListSize % 100 (IDIOM: 30).
      if (recordCount != size % expectedCount) {
        System.err.println(OAIPMHUtilitiesOnline.ERROR + ": " + recordOrHeader
            + " count mismatch, should be " + size % expectedCount + ", but is " + recordCount);
        assertTrue(false);
      }

      // No resumption token available in response.
      return "-1";
    }

    return restok;
  }

  /**
   * @param theWebClient
   * @param theVerb
   * @param theMetadataPrefix
   * @param theSet
   * @return
   * @throws IOException
   */
  protected static int testList(Client theWebClient, String theVerb, String theMetadataPrefix,
      String theSet) throws IOException {
    return testList(theWebClient, theVerb, theMetadataPrefix, theSet, OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL);
  }

  /**
   * @param theWebClient
   * @param theVerb
   * @param theMetadataPrefix
   * @param theSet
   * @param from
   * @param until
   * @return How many pages were delivered
   * @throws IOException
   */
  protected static int testList(Client theWebClient, String theVerb, String theMetadataPrefix,
      String theSet, String from, String until) throws IOException {

    int result;

    long startTime = System.currentTimeMillis();

    String testOccurance = "header";
    if (theVerb.equals(OAIPMHUtilitiesOnline.VERB_LIST_RECORDS)) {
      testOccurance = "record";
    }

    String path = "verb=" + theVerb + "&metadataPrefix=" + theMetadataPrefix;

    if (theSet != null && !theSet.isEmpty()) {
      path += "&set=" + theSet;
    }

    if (from != null && until != null) {
      path += "&from=" + from + "&until=" + until;
    }

    Response response = OAIPMHUtilitiesOnline.getHttpResponse(theWebClient, path);
    String responseString = IOUtils.readStringFromStream((InputStream) response.getEntity());
    int status = response.getStatus();

    long timeRunning = System.currentTimeMillis() - startTime;
    System.out.println("\ttime: " + OAIPMHUtilitiesOnline.getDurationInSecs(timeRunning));

    String restok = examineResumptionTokenTag(responseString, testOccurance, "", theMetadataPrefix);
    examineIdentifiers(responseString);

    result = 1;
    while (status == HttpStatus.SC_OK && !restok.equals("-1")) {
      path = "verb=" + theVerb + "&resumptionToken=" + restok;
      response = OAIPMHUtilitiesOnline.getHttpResponse(theWebClient, path);
      responseString = IOUtils.readStringFromStream((InputStream) response.getEntity());
      timeRunning = System.currentTimeMillis() - startTime;
      System.out.println("\ttime: " + OAIPMHUtilitiesOnline.getDurationInSecs(timeRunning));
      restok = examineResumptionTokenTag(responseString, testOccurance, restok, theMetadataPrefix);
      examineIdentifiers(responseString);
      result++;
    }

    System.out.println("\tpage amount: " + result);
    System.out.println(OAIPMHUtilitiesOnline.OK);

    return result;
  }

  /**
   * @param theResponseString
   * @return
   * @throws IOException
   */
  protected static void examineIdentifiers(String theResponseString) throws IOException {
    if (theResponseString.contains("<identifier>hdl:21</identifier>")) {
      System.err.println(OAIPMHUtilitiesOnline.ERROR
          + " IN OAIPMH RESPONSE: identifier tag is corrupt!" + theResponseString);
      assertTrue(false);
    }
  }

}
