package info.textgrid.middleware.test.online;

import static org.junit.Assert.assertTrue;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import javax.ws.rs.core.Response;
import org.apache.cxf.helpers.IOUtils;
import org.apache.cxf.jaxrs.client.Client;
import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.apache.http.HttpStatus;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.OAIPMHProducer;

/**
 * <p>
 * Some online tests for the DARIAH-DE Repository OAIMPH service.
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 */
@Ignore
public class TestDHOAIPMHOnline {

  // **
  // FINALS
  // **

  private static final String PROPERTIES_FILE = "oaipmh.test.repository-de-dariah-eu.properties";
  // private static final String PROPERTIES_FILE = "oaipmh.test.trep-de-dariah-eu.properties";

  // **
  // STATICS
  // **

  // Some JAXRS things.
  private static String oaipmhEndpoint;
  private static OAIPMHProducer JAXRSClient;
  private static Client oaipmhWebClient;

  // Properties
  private static String expectedListSets;
  private static String checkListIdentifiersSet;
  private static Integer checkListIdentifiersSetExpectedPages;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Load properties file.
    Properties p = new Properties();
    p.load(new FileInputStream(
        OAIPMHUtilitiesOnline.getResource(PROPERTIES_FILE)));

    System.out.println("Properties file: " + PROPERTIES_FILE);
    p.list(System.out);

    // Get other needed properties.
    oaipmhEndpoint = p.getProperty("oaipmhEndpoint");
    expectedListSets = p.getProperty("expectedListSets");
    checkListIdentifiersSet = p.getProperty("checkListIdentifiersSet");
    checkListIdentifiersSetExpectedPages =
        Integer.parseInt(p.getProperty("checkListIdentifiersSetExpectedPages").trim());

    // Get OAI-PMH REST endpoint and HTTP client.
    System.out.println(
        "Getting OAI-PMH HTTP client --> " + oaipmhEndpoint + " <--");

    // Get proxy first, set policy.
    JAXRSClient = JAXRSClientFactory.create(oaipmhEndpoint,
        OAIPMHProducer.class);
    HTTPConduit conduit = WebClient.getConfig(JAXRSClient).getHttpConduit();
    HTTPClientPolicy policy = new HTTPClientPolicy();
    policy.setReceiveTimeout(OAIPMHUtilitiesOnline.OAIPMH_CLIENT_TIMEOUT);
    conduit.setClient(policy);

    // Create Web Client from Web Proxy.
    oaipmhWebClient = WebClient.client(JAXRSClient);
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {
    //
  }

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testGetVersion() throws IOException {

    String shouldStartWith = "oaipmh-core";

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#GETVERSION for '"
        + shouldStartWith + "'");

    String versionString = JAXRSClient.getVersion();

    if (!versionString.startsWith(shouldStartWith)) {
      System.err.println(OAIPMHUtilitiesOnline.ERROR
          + ": response should start with '" + shouldStartWith + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + versionString);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testRootWithSlash() throws IOException {

    String url = "oai/";
    String shouldContain = "DARIAH-DE Repository";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testRootWithoutSlash() throws IOException {

    String url = "oai";
    String shouldContain = "DARIAH-DE Repository";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testBlubbVerb() throws IOException {

    String url = "verb=BLUBB";
    String shouldContain = "Illegal OAI verb";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testIdentify() throws IOException {

    String verb = "Identify";
    String shouldContain = "DARIAH-DE Repository";

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#IDENTIFY");

    Response response = OAIPMHUtilitiesOnline.getHttpResponse(oaipmhWebClient,
        "verb=" + verb);
    int status = response.getStatus();

    String responseString = IOUtils
        .readStringFromStream((InputStream) response.getEntity());

    if (status != HttpStatus.SC_OK
        || !responseString.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(OAIPMHUtilitiesOnline.ERROR
          + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + responseString);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListSets() throws IOException {
    testListSet(expectedListSets, expectedListSets);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListRecords", "oai_dc",
        OAIPMHUtilitiesOnline.NO_SET);
  }


  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testGetRecordDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#GETRECORD");

    String identifier = "";
    String shouldContain = "";

    String path = "verb=GetRecord&identifier=" + identifier
        + "&metadataPrefix=oai_dc";

    // FIXME Complete test!
  }

  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testGetRecordDATASITE() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#GETRECORD");

    String identifier = "";
    String shouldContain = "";

    String path = "verb=GetRecord&identifier=" + identifier
        + "&metadataPrefix=oai_datacite";

    // FIXME Complete test!
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_dc",
        OAIPMHUtilitiesOnline.NO_SET);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersSetDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS");

    int pages =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_dc",
            checkListIdentifiersSet);

    if (pages != checkListIdentifiersSetExpectedPages) {
      assertTrue(pages + " != " + checkListIdentifiersSetExpectedPages, false);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersSetDCDifferentPrefixes() throws IOException {

    String uri = checkListIdentifiersSet;

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS - HDL PREFIX");

    if (!uri.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      assertTrue("missing '" + OAIPMHUtilitiesOnline.HDL_PREFIX + "' prefix!", false);
    }
    System.out.println("uri: " + uri);
    int pages1 = OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_dc", uri);
    if (pages1 != checkListIdentifiersSetExpectedPages) {
      System.out.println(pages1 + " != " + checkListIdentifiersSetExpectedPages);
    }

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS - NO PREFIX");

    if (checkListIdentifiersSet.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      uri = checkListIdentifiersSet.substring(4);
    }
    System.out.println("uri: " + uri);
    int pages2 = OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_dc", uri);
    if (pages2 != checkListIdentifiersSetExpectedPages) {
      System.out.println(pages2 + " != " + checkListIdentifiersSetExpectedPages);
    }

    // Check pages count.
    if (pages1 != checkListIdentifiersSetExpectedPages
        || pages2 != checkListIdentifiersSetExpectedPages) {
      assertTrue("pages count does not match", false);
    }
  }


  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testListIdentifiersSetDCStarPrefixe() throws IOException {

    // TODO Un-ignore if identifier check is implemented!

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS - STAR PREFIX");

    if (checkListIdentifiersSet.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      checkListIdentifiersSet =
          OAIPMHUtilitiesOnline.STAR_PREFIX + checkListIdentifiersSet.substring(4);
    }
    System.out.println("uri: " + checkListIdentifiersSet);
    int pages3 =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_dc",
            checkListIdentifiersSet);
    if (pages3 != checkListIdentifiersSetExpectedPages) {
      System.out.println(pages3 + " != " + checkListIdentifiersSetExpectedPages);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersDATASITE() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTIDENTIFIERS");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, "ListIdentifiers", "oai_datacite",
        OAIPMHUtilitiesOnline.NO_SET);
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * @param theSet
   * @throws IOException
   */
  private static void testListSet(String theSet, String theExpectedResponse)
      throws IOException {

    String verb = "ListSets";

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTSETS");

    Response response = OAIPMHUtilitiesOnline.getHttpResponse(oaipmhWebClient,
        "verb=" + verb);
    int status = response.getStatus();

    String responseString = IOUtils
        .readStringFromStream((InputStream) response.getEntity());

    if (status != HttpStatus.SC_OK
        || !responseString.contains(theExpectedResponse)) {
      System.err.println("\tstatus: " + status);
      System.err.println(
          OAIPMHUtilitiesOnline.ERROR + ": response should contain '" + theExpectedResponse + "'");
      System.err.println(responseString);
      assertTrue(false);
    }

    System.out.println("\tresponse: " + responseString);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @param theURL
   * @param shouldContain
   * @throws IOException
   */
  private static void testRootURL(String theURL, String shouldContain)
      throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#ROOTURL");

    Response response = OAIPMHUtilitiesOnline.getHttpResponse(oaipmhWebClient,
        theURL);
    int status = response.getStatus();

    String responseString = IOUtils
        .readStringFromStream((InputStream) response.getEntity());

    if (status != HttpStatus.SC_OK || !responseString.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err
          .println(
              OAIPMHUtilitiesOnline.ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + responseString);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

}
